// Class: RowMajorGridIterator
//
// Author: Alyce Brady
//
// License Information:
//   This class is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation.
//
//   This class is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

import edu.kzoo.grid.BoundedGrid;
import edu.kzoo.grid.Location;

/**
 *  Grid Iterator Lab:<br>
 *
 *    A RowMajorGridIterator object provides an iterator that
 *    steps through the locations of a BoundedGrid object in
 *    row-major order.  It iterates through the locations and
 *    processes the rows of the grid, from 0 to R - 1, where
 *    R is the number of rows in the grid.  As part of processing
 *    each row, it traverses through the columns of that row,
 *    from 0 to C - 1, where C is the number of columns in the
 *    grid.
 *
 *  @author Alyce Brady
 *  @version 24 March 2004
 *  @see edu.kzoo.grid.Grid
 *  @see edu.kzoo.grid.Location
 **/
public class RowMajorGridIterator extends GridIterator
{
    /** Constructs an iterator object that steps through a
     *  grid.
     *  @param grid     grid to iterate through
     **/
    public RowMajorGridIterator(BoundedGrid grid)
    {
        super(grid);
    }

    /** Helper Method: Finds next location for iterator.
     **/
    protected Location findNextLocation()
    {
        // Get the "current" next location.
        Location loc = nextLocation();

        // Find the location after the "current" next location.
        int nextCol = loc.col() + 1;
        if ( nextCol < grid().numCols() )
            return new Location(loc.row(), nextCol);
        else
            return new Location(loc.row() + 1, 0);
    }

}
